/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is the Dict
 *
 * The Initial Developer of the Original Code is David Costanzo
 * Portions created by the Initial Developer are Copyright (C) 2003-2005
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */
"use strict";

Components.utils.import("chrome://dict/content/dictclient.js");

if ("undefined" == typeof(DictExtension)) {
    var DictExtension = {};
};

// Returns the selected text as a string.
DictExtension.getSelection = function(document)
{
    var node = document.popupNode;

    var selection;

    var nodeLocalName = node.localName.toUpperCase();

    if ((nodeLocalName == "TEXTAREA") ||
        (nodeLocalName == "INPUT" && node.type == "text")) {

        // The selection is either in a TEXTAREA of an INPUT HTML node.
        // This includes the text area of Mozilla Mail.
        selection = node.value.substring(
            node.selectionStart,
            node.selectionEnd);
    }
    else {
        // Get the selection page using XPCNativeWrapper.
        var winWrapper = new XPCNativeWrapper(
            document.commandDispatcher.focusedWindow,
            'getSelection()');

        selection = winWrapper.getSelection();
    }

    var selectedWord = selection.toString();

    // Sanitize the selection by removing leading an trailing non-word characters.
    // NOTE: We cannot use \W because that matches many Unicode word characters.
    // Instead, we explicitly spell out the range of non-word ASCII-7 characters.
    selectedWord = selectedWord.replace(/^[\x00-\x2F\x3A-\x40\x5B-\x60\x7B-\x7F]+/, ''); // remove leading non-alnums
    selectedWord = selectedWord.replace(/[\x00-\x2F\x3A-\x40\x5B-\x60\x7B-\x7F]+$/, ''); // remove trailing non-alnums
    selectedWord = selectedWord.replace(/\s+/g, ' '); // conflate whitespace runs

    return selectedWord;
}


DictExtension.defineSelection = function(window)
{
    var selectedWord = DictExtension.getSelection(window.document);
    DictExtension.defineWord(selectedWord, window);
}


// Initialize the context menu in the calling XUL code.
DictExtension.contextMenuInit = function()
{
    // Only show the Define Selection item when something is selected.
    var onPopup = function() {

        var item = document.getElementById('dict-extension-define-selection');

        var selection = DictExtension.getSelection(document);
        if (selection != null && selection.length != 0) {

            const LENGTHCUTOFF = 15;

            if (LENGTHCUTOFF < selection.length) {
                // selection is too long to appear in menu.  It must be abbreviated.
                selection = selection.slice(0, LENGTHCUTOFF) + '...';
            }

            var dictStringBundle = document.getElementById("dict-extension-string-bundle");
            var labelTemplate = dictStringBundle.getString("dict-extension-search-label");

            item.setAttribute("label", labelTemplate.replace('%S', selection));
            item.hidden = false; // show the menu item
        }
        else {
            item.hidden = true; // hide the menu item
        }
    }

    var elementIds = [
        "contentAreaContextMenu", // included from context.xul
        "msgComposeContext",      // included from message-overlay.xul for Thunderbird Composer
        "mailContext",            // included from message-overlay.xul for Thunderbird Reader
    ];

    for (var i in elementIds) {
        var contextMenu = document.getElementById(elementIds[i]);
        if (contextMenu != null) {
            contextMenu.addEventListener("popupshowing", onPopup, false);
        }
    }
}

// DictExtension.contextMenuInit will be called every time a new browser window is created
window.addEventListener("load", DictExtension.contextMenuInit, false);

